# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## PREPARE DATA ================================================================

# Description:
#     Collate and harmonise epidemiological, climatic and demographic data

# Paper:
#     Compound and cascading effects of climatic extremes on dengue risk in the 
#     Caribbean: An impact-based modelling framework using long- and short-lag
#     interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in epi, climate and population datasets
epi_data <- read.csv("data/barbados_denv_data_1999-2022.csv")
climate_data <- read.csv("data/barbados_climate_data_011981-052023.csv")
pop_data <- read.csv("data/barbados_pop_data_1960-2022.csv")

# set start and end years
start_year <- min(epi_data$year)
end_year <- max(epi_data$year)

# remove unused climate data columns
climate_data <- climate_data %>%
  dplyr::select(date, year, month, tas, tasmax, tasmin, tas.3mon, tasmax.3mon,
                tasmin.3mon, spi1, spi3, spi6, spi12)


## Calculate temp anomaly ------------------------------------------------------

# find temperature columns
tmp_cols <- grep("tas", colnames(climate_data))

# store means in outputs for later interpretation
# (mean reference period: jun 1981 to may 2022)
tmp_mean <- colMeans(climate_data[climate_data$date >= "1981-06-01" &
                                  climate_data$date <= "2022-05-01", tmp_cols],
                     na.rm=TRUE)
saveRDS(tmp_mean, "outputs/temperature_means.rds")

# calculate temperature anomaly by subtracting mean
for (c in tmp_cols){
  mean <- mean(climate_data[, c], na.rm=TRUE)
  climate_data[, c] <- round(climate_data[, c] - mean, 3)
}


## Create lagged variables -----------------------------------------------------

# number of columns in climate_data with temporal information
cols <- 3

# create variable list
vars <- colnames(climate_data[(cols + 1):length(climate_data)])

# create lagged variables
for (v in vars){
  # create lags for each variable
  lags <- cbind(climate_data[, 1:cols], tsModel::Lag(climate_data[[v]],
                                                     k = 0:nlag))
  lags <- lags[lags$year >= start_year,]
  colnames(lags)[(cols + 1):length(lags)] <- paste(v, 0:nlag, sep=".")
  
  # merge lagged variables with climate_data
  climate_data <- merge(climate_data, lags, by = c("date","year","month"),
                        all.x = TRUE)
  
  # remove original variable
  c <- grep(v, colnames(climate_data))[1]
  climate_data <- climate_data[, -c]
}

# remove .0 after 0-lag variables
colnames(climate_data) <- gsub(".0", "", colnames(climate_data))


## Merge epi and climate data --------------------------------------------------

# remove climate start year entries
climate_data <- climate_data[climate_data$year >= start_year, ]

# merge epi_data and climate_data
data <- merge(epi_data, climate_data, by=c("date","year","month"), all.x = TRUE)


## Merge population data -------------------------------------------------------

# remove pop start year entries
pop_data <- pop_data[pop_data$year >= start_year, ]

# merge epi_data and climate_data
data <- merge(data, pop_data, by = c("year"), all.x = TRUE)


## Calculate dengue incidence rate ---------------------------------------------

# calculate dengue incidence rate (dir)
data$dir <- round((data$cases / data$population) * 10^5, 3)

# create dir lagged by 4 months
# (i.e. latest possible dir available when forecast issued)
data$dir.4 <- round(tsModel::Lag(data$dir, k = 4), 3)

# create log of dir lagged by 4 months
data$logdir.4 <- round(log(data$dir.4 + 1), 3)


## Format data -----------------------------------------------------------------

# re-order columns
data <- data %>%
  relocate(date) %>%
  relocate(population, .after = cases) %>%
  relocate(dir, .after = population) %>%
  relocate(dir.4, .after = dir) %>%
  relocate(logdir.4, .after = dir.4)

# order data by year, month
data <- data[with(data, order(cal_year, cal_month)),]
row.names(data) <- NULL

# convert date column to date object
data$date <- as.Date(data$date)

# save data
write.csv(data, "data/barbados_data_1999-2022.csv", row.names = FALSE)


## Amend annual cycle (Jun-May) ------------------------------------------------

# rename current year and month columns
colnames(data) <- gsub("month", "cal_month", colnames(data))
colnames(data) <- gsub("year", "cal_year", colnames(data))

# set season start month
startmonth <- 6

# find number of distinct seasons
nyear <- length(unique(data$cal_year)) - 1

# subset data to start from season start month
data <- data[startmonth:(nrow(data) - (12 - startmonth + 1)), ]

# create new month column to reflect month of dengue season
data$month <- rep(1:12, nyear)

# create new year index column to define each dengue season
data$year_index <- rep(1:nyear, each = 12)

# create time column to track month of time series
data$time <- 1:nrow(data)


## Save data -------------------------------------------------------------------

# save data
write.csv(data, "data/barbados_data.csv", row.names = FALSE)


## END