# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## DATA EXPLORATION ============================================================

# Description:
#     Explore and visualise epidemiological and climatic data

# Paper:
#     Compound and cascading effects of climatic extremes on dengue risk in the 
#     Caribbean: An impact-based modelling framework using long- and short-lag
#     interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in harmonised data
data <- read.csv("data/barbados_data.csv")
tmp_means <- readRDS("outputs/temperature_means.rds")

# format dates
data$date <- as.Date(data$date)
date_start <- min(data$date)
date_end <- max(data$date)

# read in nino34 data
nino <- read.csv("data/nino_anom_ref_1991-2020.csv")
nino <- nino[, c(1, 2, 6, 7)]
colnames(nino) <- c("cal_year", "cal_month", "nino34", "oni")
data <- merge(data, nino, by = c("cal_year", "cal_month"), x.all = TRUE)

# re-order data
data <- data[with(data, order(cal_year, cal_month)),]
rownames(data) <- NULL


## Explore data ----------------------------------------------------------------

# find peak months of confirmed cases per dengue season
annual_peak <- data %>%
  group_by(year_index) %>%
  slice(which.max(cases)) %>%
  select(date, cal_year, cal_month, cases, month, year_index)

# calculate mean confirmed cases per calendar month
average_month <- data %>%
  group_by(month) %>%
  summarise(cases=mean(cases, na.rm = TRUE)) %>%
  as.data.frame()

# plot mean confirmed cases per calendar month
plt_average_month_cases <- ggplot() +
  geom_line(data = average_month, aes(month, cases)) +
  ggtitle("Average Dengue Cases per Month in Barbados (Jun-1999 to May-2022)") +
  xlab("Month of Year") +
  scale_x_continuous(breaks=seq(1, 12, by = 1), expand = c(.03,.03),
                     labels=c("Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
                              "Jan", "Feb", "Mar", "Apr", "May")) +
  ylab("Dengue Cases") +
  scale_y_continuous(breaks = seq(0, 50, by = 10), expand = c(.03, .03),
                     limits = c(0, 50)) +
  theme_bw(base_size = 12) +
  theme(panel.border = element_blank(), plot.title = element_text(hjust=0.5))
plt_average_month_cases

# plot boxplot of confirmed cases per month
plt_month_cases_boxplot <- ggplot(data, aes(x = as.factor(month), y = cases)) +
  geom_boxplot(outlier.shape = NA) +
  ggtitle("Average Dengue Cases per Month in Barbados (Jun-1999 to May-2022)") +
  xlab("Month of Year") +
  scale_x_discrete(breaks = seq(1, 12, by = 1), expand = c(.03, .03),
                   labels=c("Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
                            "Jan", "Feb", "Mar", "Apr", "May")) +
  ylab("Dengue Cases") +
  scale_y_continuous(breaks = seq(0, 200, by=20), expand = c(.03, .03)) +
  coord_cartesian(ylim = c(0, 200)) +
  theme_bw(base_size = 12) +
  theme(panel.border = element_blank(), plot.title = element_text(hjust=0.5))
plt_month_cases_boxplot


## Plot time series ------------------------------------------------------------

# find moving monthly 50th, 75th and 95th percentiles of dengue cases
monthly_50pct <- c()
monthly_75pct <- c()
monthly_95pct <- c()

# calculate percentiles per month
for (m in 1:12){
  
  # calculate monthly percentiles
  p50 <- quantile(data[(data$month == m), "cases"], p = 0.50)[[1]]
  p75 <- quantile(data[(data$month == m), "cases"], p = 0.75)[[1]]
  p95 <- quantile(data[(data$month == m), "cases"], p = 0.95)[[1]]
  
  # append to lists
  monthly_50pct <- c(monthly_50pct, p50)
  monthly_75pct <- c(monthly_75pct, p75)
  monthly_95pct <- c(monthly_95pct, p95)
}

data$cases_50pct <- rep(monthly_50pct, max(data$year_index))
data$cases_75pct <- rep(monthly_75pct, max(data$year_index))
data$cases_95pct <- rep(monthly_95pct, max(data$year_index))

# plot national monthly dengue cases
# (select which geom_line to plot concurrently)

colors <- c("50th percentile" = "yellow4", "75th percentile" = "darkorange2",
            "95th percentile" = "red")

plt_nat_month_cases_threshold <- ggplot() +
  geom_line(data = data, aes(date, cases)) +
  #geom_line(data = data, aes(date, cases_50pct, color = "50th percentile")) +
  geom_line(data = data, aes(date, cases_75pct, color = "75th percentile")) +
  #geom_line(data = data, aes(date, cases_95pct, color = "95th percentile")) +
  ggtitle("Monthly Dengue Cases in Barbados (Jun-1999 to May-2022)") +
  xlab("Month of Year") +
  scale_x_date(limit = c(date_start, date_end), expand = c(0,0),
               date_labels = "%b %Y", date_breaks = "6 months") +
  ylab("Dengue Cases") +
  scale_y_continuous(breaks=seq(0, 300, by = 20), expand = c(.03,.03)) +
  labs(color = "") +
  scale_color_manual(values = colors) +
  theme_bw(base_size = 12) +
  theme(panel.border = element_blank(), plot.title = element_text(hjust=0.5),
        axis.text.x = element_text(angle=60, hjust=1), legend.position = "top")
plt_nat_month_cases_threshold


## Create facet plot -----------------------------------------------------------

# facet plot time series for dengue, mean temp, spi6 and nino34
long_data <- data
long_data$tas <- long_data$tas + tmp_means[["tas"]]
long_data <- long_data %>%
  pivot_longer(cols = c(cases, tas, spi6, nino34), 
               names_to = "Variable", 
               values_to = "Value")

plt_facet <- ggplot(long_data, aes(x = as.Date(date), y = Value)) +
  geom_line(aes(color = Variable)) +
  scale_color_manual(values = c(cases = "black", tas = "red3", spi6 = "blue",
                                nino34 = "purple4")) +
  facet_wrap(~Variable, scales = "free_y", ncol = 1,
             labeller = as_labeller(c(cases = "Dengue Cases",
                                      tas = "Mean Temp (°C)",
                                      spi6 = "SPI-6",
                                      nino34 = "Niño 3.4 SSTA (°C)")),
             strip.position = "left") +
  labs(x = "Month of Year", y = NULL) +
  scale_x_date(limit=c(date_start, date_end), expand = c(0, 0),
               date_labels = "%b %Y", date_breaks = "6 months") +
  theme_bw(base_size = 12) +
  theme(panel.border = element_rect(linewidth = 0),
        axis.text.x = element_text(angle = 45, hjust = 1),
        legend.position = "none", strip.background = element_blank(),
        strip.placement = "outside", plot.title = element_text(hjust = 0.5))
#ggsave("outputs/figs/SF_02.png", width = 12, height = 7, dpi = 300)


## Plot binary outbreaks -------------------------------------------------------

# calculate outbreak months
data$outbreak <- ifelse(data$cases - data$cases_75pct > 0, TRUE, FALSE)

# plot outbreaks TRUE/FALSE
plt_nat_outbreak_bin <- ggplot(data, aes(date, outbreak)) +
  geom_point() +
  ggtitle("Dengue Outbreak per Month in Barbados (Jun-1999 to May-2022)") +
  xlab("Month of Year") +
  scale_x_date(limit = c(date_start, date_end), expand = c(0, 0),
               date_labels = "%b %Y", date_breaks = "6 months") +
  ylab("Dengue Outbreak") +
  theme_bw(base_size = 12) +
  theme(panel.border = element_blank(), plot.title = element_text(hjust = 0.5),
        axis.text.x = element_text(angle = 60, hjust = 1))
plt_nat_outbreak_bin


## Plot heat maps --------------------------------------------------------------

# create month and year labels
month.lab <- c(month.abb[c(6:12)], month.abb[c(1:5)])
year.lab <- paste(seq(min(data$cal_year), max(data$cal_year) - 1, 1),
                  stringr::str_sub(seq(min(data$cal_year) + 1, max(data$cal_year),
                                       1), -2, -1), sep="/")

# heat map of dengue cases
denv_heat <- ggplot(data = data, aes(x = month, y = year_index, fill = cases)) + 
  geom_raster() +
  ylab("Year") + 
  xlab("Month") +
  scale_fill_gradientn(name = "Cases", colours = brewer.pal(9, "PuRd")) + 
  scale_y_continuous(breaks = seq(1, max(data$year_index), 1),
                     labels = year.lab, expand = expansion(0)) +
  scale_x_continuous(breaks = c(1:12), labels = month.lab, expand = expansion(0)) +
  theme_bw()
denv_heat

# heat map of mean temperature anomaly
tmp <- data
tmp$tas <- tmp$tas + tmp_means["tas"]

tas_heat <- ggplot(data = tmp, aes(x = month, y = year_index, fill = tas)) + 
  geom_raster() +
  ylab("Year") + 
  xlab("Month") +
  scale_fill_viridis(name = "Mean Temp", option = "magma", direction = -1) +
  scale_y_continuous(breaks = seq(1, max(data$year_index), 1),
                     labels = year.lab, expand = expansion(0)) +
  scale_x_continuous(breaks = c(1:12), labels = month.lab, expand = expansion(0)) +
  theme_bw()
tas_heat

# heat map of spi6
spi6_heat <- ggplot(data = data, aes(x = month, y = year_index, fill = spi6)) + 
  geom_raster() +
  ylab("Year") + 
  xlab("Month") +
  scale_fill_gradientn(name = "SPI-6", colours = brewer.pal(11, "BrBG")) +
  scale_y_continuous(breaks = seq(1, max(data$year_index), 1),
                     labels = year.lab, expand = expansion(0)) +
  scale_x_continuous(breaks = c(1:12), labels = month.lab, expand = expansion(0)) +
  theme_bw()
spi6_heat

# heat map of nino34
nino_heat <- ggplot(data = data, aes(x = month, y = year_index, fill = nino34)) + 
  geom_raster() +
  ylab("Year") + 
  xlab("Month") +
  scale_fill_gradientn(name = "ONI", colours = rev(brewer.pal(11, "RdBu")),
                       breaks=c(-3,0,3), limits=c(-3,3)) +
  scale_y_continuous(breaks = seq(1, max(data$year_index), 1),
                     labels = year.lab, expand = expansion(0)) +
  scale_x_continuous(breaks = c(1:12), labels = month.lab, expand = expansion(0)) +
  theme_bw()
nino_heat

# combine heat maps
ggarrange(denv_heat, tas_heat, spi6_heat, nino_heat, 
          ncol = 2, nrow = 2, labels = c("A", "B", "C", "D"), align = "hv",
          hjust = -1, vjust = 1, font.label = list(size = 14, face = "plain"))


## Evaluate correlation --------------------------------------------------------

# select response (cases) and covariates (excl. lags)
data_cor <- data %>%
  select(c(4, 9, 16, 23, 30, 37, 44, 51, 58, 65, 72)) %>%
  drop_na()

# look for correlations
cor(data_cor)

# simplify selection
# (final variables: cases, tas.3mon.4, spi6.5, spi6.1)
data_cor_simple <- data %>%
  select(c(4, 34, 66, 70)) %>%
  drop_na()

# look for correlations
cor(data_cor_simple)


## END
