# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## CROSS VALIDATION ============================================================

# Description:
#     Remove 12 months out of the sample around each month in the time series to
#     predict dengue cases "out of sample" via leave one out cross validation.
#     Perform this model verification for all candidate models.

# Paper:
#     Compound and cascading effects of climatic extremes on dengue risk in the 
#     Caribbean: An impact-based modelling framework using long- and short-lag
#     interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in harmonised data
data <- read.csv("data/barbados_data.csv")

# load model output
spi6.mod.out <- readRDS("outputs/spi6_mods.rds")
candidate_mods <- c(23,29,30,58,59,138,173)


## Set model parameters --------------------------------------------------------

# set start year, start month and start index
startyear <- data$cal_year[1]
startmonth <- data$cal_month[1]
startid <- nrow(data) - 12 * 10 + 1  # last 10 years of dataset

# model covariates
vars.b <- c(NA)
vars.p <- spi6.mod.out$mod.gof$vars[candidate_mods]

# forecast lead time
lags.b <- c(3)
lags.p <- rep(3, length(vars.p))

# random effects
re1 <- paste("f(month, model = 'rw2', cyclic = TRUE, constr = TRUE,",
             "scale.model = TRUE, hyper = precision.prior)")
re2 <- "f(year_index, model = 'rw2', hyper = precision.prior)"
fe <- "logdir.4"

res.b <- list(re1)
res.p <- list(re1, re2, fe)


## Run cross validation --------------------------------------------------------

# run cross validation for baseline model
# source("functions/out_of_sample_progressive_adjusted.R")
# pred.b <- out_of_sample(data = data, response = "cases", save = T, 
#                         vars = vars.b, res = res.b, lags = lags.b,
#                         start_id = startid, source_path = "functions",
#                         file_path = "outputs", file_name = "preds_b")

# run cross validation for candidate models
# scheme 1: moving 12-month window with issue month 8 and target month 12
# source("functions/out_of_sample_progressive_adjusted.R")
# pred.i.ay <- out_of_sample(data = data, response = "cases", save = T,
#                            vars = vars.p, res = res.p, lags = lags.p,
#                            start_id = startid, source_path="functions",
#                            file_path = "outputs", file_name = "preds_i_adjustedyear_logdir")

# run cross validation for candidate models
# scheme 2: remove yearly effect from REs
# source("functions/out_of_sample_progressive_adjusted.R")
# pred.i.ny <- out_of_sample(data = data, response = "cases", save = T,
#                            vars = vars.p, res = res.b, lags = lags.p, 
#                            start_id = startid, source_path="functions",
#                            file_path = "outputs", file_name = "preds_i_noyeareffect_logdir")

# run cross validation for candidate models
# scheme 3: set year index to NA
# source("functions/out_of_sample_progressive.R")
# pred.i.na <- out_of_sample(data = data, response = "cases", save = T,
#                            vars = vars.p, res = res.p, lags = lags.p, 
#                            start_id = startid, source_path="functions",
#                            file_path = "outputs", file_name = "preds_i_nayear_logdir")

pred.b <- readRDS("outputs/prediction_outputs/preds_b.rds")
pred.i.ay <- readRDS("outputs/prediction_outputs/preds_i_adjustedyear_logdir.rds")
pred.i.ny <- readRDS("outputs/prediction_outputs/preds_i_noyeareffect_logdir.rds")
pred.i.na <- readRDS("outputs/prediction_outputs/preds_i_nayear_logdir.rds")


## Tabulate results ------------------------------------------------------------

# tabulate hit rate, false alarm rate, auc, trigger and crps
tab <- rbind(pred.b$predictive_ability, pred.i.ay$predictive_ability,
             pred.i.ny$predictive_ability, pred.i.na$predictive_ability)
tab$model <- c("current practice",
               paste(rep(c("Adj. yr:", "no YE:", "NA yr:"),
                         each = length(vars.p)), vars.p))
row.names(tab) <- NULL


## Plot final selected model ---------------------------------------------------

# select pred.i
pred.i <- pred.i.ay

# final model selected
i <- 4

# plot observed vs fitted
g1 <- plot.fit.cv(pred.i, startyear, startmonth, i)

# plot outbreak probability
g2 <- plot.prob.outbreak.cv(pred.i, data[startid, "cal_year"], startmonth, i,
                            threshold = pred.i$predictive_ability[i, "trigger"])

# plot double roc curve with current practice model
rocplt <- roc(pred.i[["outbreak_probability"]][[i]][["obs_outbreak"]],
              pred.i[["outbreak_probability"]][[i]][["pred_outbreak_prob"]],
              direction = "<", ci = TRUE, quiet = TRUE)
ciobj <- ci.se(rocplt, specificities = seq(0, 1, l=100))
ciobj <- data.frame(x = as.numeric(rownames(ciobj)),
                    lower = ciobj[,1], upper = ciobj[,3])

rocplt2 <- roc(pred.b[["outbreak_probability"]][[1]][["obs_outbreak"]],
               pred.b[["outbreak_probability"]][[1]][["pred_outbreak_prob"]],
               direction = "<", ci = TRUE, quiet = TRUE)
ciobj2 <- ci.se(rocplt2, specificities=seq(0, 1, l=100))
ciobj2 <- data.frame(x = as.numeric(rownames(ciobj2)),
                    lower = ciobj2[,1], upper = ciobj2[,3])

g3 <- plot_roc_2(rocplt, rocplt2, ciobj, ciobj2, "")

# assemble as single plot
ggdraw() +
  draw_plot(g1, x = 0, y = .6, width = 1, height = .4) +
  draw_plot(g2, x = 0, y = 0, width = .5, height = .6) +
  draw_plot(g3, x = .5, y = 0, width = .5, height = .6) +
  draw_plot_label(label = c("A", "B", "C"), size = 18,
                  x = c(0, 0, .5), y = c(1, .58, .58)) +
  theme_void() +
  theme(plot.background = element_rect(fill = "white")) +
  panel_border(remove = TRUE)
#ggsave("outputs/figs/F_03.png", dpi=600, height=12, width=15, bg="white")


## Compare interaction vs additive model ---------------------------------------

# run additive model
vars.p <- c(gsub("\\*", "+", spi6.mod.out$mod.gof$vars[candidate_mods[i]]))
# source("functions/out_of_sample_progressive_adjusted.R")
# pred.a <- out_of_sample(dt=data, response="cases", spatial=FALSE,
#                         vars=vars.p, res=res.p, lags=lags.p[1], start_id=startid,
#                         source_path="functions", save=T,
#                         file_path="outputs", file_name="preds_a_adjustedyear_best")

# calculate true positive, false positive, true negative, false negative rates
pred.a <- readRDS("outputs/prediction_outputs/preds_a_adjustedyear_best.rds")

pred.fm <- pred.i$outbreak_probability[[i]]
pred.bl <- pred.b$outbreak_probability[[1]]
pred.am <- pred.a$outbreak_probability[[1]]

pred.fm$alert <- pred.fm$pred_outbreak_prob >= pred.i$predictive_ability[i, "trigger"]
pred.bl$alert <- pred.bl$pred_outbreak_prob >= pred.b$predictive_ability[1, "trigger"]
pred.am$alert <- pred.am$pred_outbreak_prob >= pred.a$predictive_ability[1, "trigger"]

signal <- data.frame(model = rep(c("Interaction", "Additive", "Baseline"),
                                 each=4),
                     condition = rep(c("4 Hit", "3 Correct Rejection",
                                   "2 False Alarm", "1 Miss"), 3),
                     value = c(
                       length(pred.fm$alert[pred.fm$alert == 1 &
                                              pred.fm$obs_outbreak == 1]),
                       length(pred.fm$alert[pred.fm$alert == 0 &
                                              pred.fm$obs_outbreak == 0]),
                       length(pred.fm$alert[pred.fm$alert == 1 &
                                              pred.fm$obs_outbreak == 0]),
                       length(pred.fm$alert[pred.fm$alert == 0 &
                                              pred.fm$obs_outbreak == 1]),
                       length(pred.am$alert[pred.am$alert == 1 &
                                              pred.am$obs_outbreak == 1]),
                       length(pred.am$alert[pred.am$alert == 0 &
                                              pred.am$obs_outbreak == 0]),
                       length(pred.am$alert[pred.am$alert == 1 &
                                              pred.am$obs_outbreak == 0]),
                       length(pred.am$alert[pred.am$alert == 0 &
                                              pred.am$obs_outbreak == 1]),
                       length(pred.bl$alert[pred.bl$alert == 1 &
                                              pred.bl$obs_outbreak == 1]),
                       length(pred.bl$alert[pred.bl$alert == 0 &
                                              pred.bl$obs_outbreak == 0]),
                       length(pred.bl$alert[pred.bl$alert == 1 &
                                              pred.bl$obs_outbreak == 0]),
                       length(pred.bl$alert[pred.bl$alert == 0 &
                                              pred.bl$obs_outbreak == 1])))
signal[,"value"] <- round(signal[,"value"] / 120 * 100, 1)

cols <- c("#AA1155", "#F8B4D3", "#B4E4DA", "#369683")

ggplot(data=signal, aes(x=factor(model,
                                 levels=c("Baseline", "Additive",
                                          "Interaction")),
                        y=value, fill=condition)) +
  geom_bar(stat="identity") +
  scale_fill_manual("", values = cols, labels = c("False Negative", 
                                                  "False Positive",
                                                  "True Negative",
                                                  "True Positive")) +
  xlab("Model Formulation") +
  ylab("Percentage of Forecasts (%)") +
  theme_bw() + 
  theme(text = element_text(size = 16),
        plot.background = element_rect(fill = "white"),
        panel.border = element_blank(), axis.ticks.x=element_blank(),
        axis.title.y = element_text(margin = margin(t = 0, r = 10, b = 0, l = 0)),
        axis.title.x = element_text(margin = margin(t = 20, r = 0, b = 0, l = 0)))
#ggsave("outputs/figs/F_04.png", dpi=600, height=12, width=12)


## END