# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## LOAD PACKAGES AND FUNCTIONS =================================================

# Description:
#     This script contains most of the packages and functions with the long- and
#     short-lag interaction repository.

# Paper:
#     Compound effects of climatic extremes on dengue risk in the Caribbean: An
#     impact-based modelling framework using long- and short-lag interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Packages --------------------------------------------------------------------

# specify packages
packages <- c("RColorBrewer", "geofacet", "ggpubr", "ggthemes", "ggtext", "zoo",  
              "leaflet", "reactable", "viridis", "cowplot", "pROC", "lubridate", 
              "glue", "sf", "tsModel", "INLA", "tidyverse", "readxl")

# install packages
# install.packages("INLA", 
#                  repos=c(getOption("repos"),
#                          INLA="https://inla.r-inla-download.org/R/testing"),
#                  dep=TRUE)
# lapply(packages[2:length(packages)], install.packages, character.only = TRUE)

# load packages
lapply(packages, library, character.only = TRUE)


## Variables -------------------------------------------------------------------

# set maximum lag
nlag = 6

# define priors
precision.prior <- list(prec = list(prior = "pc.prec", param = c(0.5, 0.01)))


## Data processing functions ---------------------------------------------------

# use year and month columns to add date column
monthyeartodate <- function(data, year_col="year", month_col="month") {
  # add a month-year column for plotting
  data$date <- zoo::as.yearmon(paste(data[[year_col]], data[[month_col]]),
                               "%Y %m")
  # move column to first in dataframe
  data <- data %>% relocate(date) %>%
    mutate(date=as.Date(date, format="%d-%m-%Y"))
  
  return(data)
}

# convert date to year and month column format
datetomonthyear <- function(data_frame) {
  # create columns year/month as integers from date column
  # move to first two columns and drop date
  data_frame_monthly <- data_frame %>%
    mutate(year = as.integer(format(date, "%Y")),
           month = as.integer(format(date, "%m"))) %>%
    relocate(date, year, month)
  return(data_frame_monthly)
}


## Goodness of fit functions ---------------------------------------------------

# bayesian r squared function
rsq <- function(model, null){
  dev_model <- model[["dic"]][["deviance.mean"]]
  dev_null <- null[["dic"]][["deviance.mean"]]
  n <- nrow(model[["summary.fitted.values"]])
  
  rsq <- 1 - exp((-2 / n) * ((dev_model / -2) - (dev_null / -2)))
  return(round(rsq, 3))
}

# deviance information criterion (dic)
dic <- function(model){
  dic <- round(model[["dic"]][["dic"]],0)
  return(round(dic, 0))
}

# difference in dic between model and baseline
dic.diff <- function(model, null){
  dic_model <- dic(model)
  dic_null <- dic(null)
  
  diff <- dic_null - dic_model
  return(diff)
}

# Watanabe–Akaike information criterion (waic)
waic <- function(model){
  waic <- round(model[["waic"]][["waic"]],0)
  return(round(waic, 0))
}

# difference in waic between model and baseline
waic.diff <- function(model, null){
  waic_model <- waic(model)
  waic_null <- waic(null)
  
  diff <- waic_null - waic_model
  return(diff)
}

# mean absolute error (mae)
mae <- function(obs, model){
  fit <- model[["summary.fitted.values"]][["0.5quant"]]
  mae <- mean(abs(fit-obs), na.rm=TRUE)
  return(round(mae, 3))
}

# difference in mae between model and baseline
mae.diff <- function(obs, model, null){
  mae_model <- mae(obs, model)
  mae_null <- mae(obs, null)
  
  diff <- mae_null - mae_model
  return(diff)
}


## Model fitting functions -----------------------------------------------------

# run model in INLA
runinlamod <- function(formula, data = data, family = "nbinomial",
                       config = FALSE){
  # formula     :  formula for INLA
  # data        :  data as dataframe object
  # family      :  likelihood distribution (default: negative binomial)
  # config      :  enable sampling (default: FALSE, set to TRUE in final runs)
  mod <- inla(formula = formula, data = data, family = family,
              offset = log(population/10^5),
              control.inla = list(strategy = 'adaptive'), 
              control.compute = list(dic = TRUE, waic = TRUE, config = config,
                                     cpo = TRUE, return.marginals = FALSE),
              control.fixed = list(correlation.matrix = TRUE, 
                                   prec.intercept = 1, prec = 1),
              control.predictor = list(link = 1, compute = TRUE), 
              verbose = FALSE)
  mod <- inla.rerun(mod)
  return(mod)
}

# update base formula with covariates
form.update <- function(x){
  form <- paste(baseformula, x, sep=" + ")
  return(form)
}

# create mod.out list for fit.models function below
create.mod.out <- function(mod.list){
  #  input:
  #    -  mod.list : dataframe w/ single column containing variable combinations
  #                  (representing fixed effects to be run in models)
  #  output:
  #    list (mod.out) with elements:
  #    -  mod.gof  : dataframe of GOF metrics (vars, dic, dic_vs_base, waic,
  #                                            waic_vs_base, mae, mae_vs_base,
  #                                            rsq, formula)
  #    -  fitted   : list of fitted values (length = number of models)
  #    -  fixed    : list of fixed effects (length = number of models)
  #    -  random   : list of random effects (length = number of models)
  mod.out <- list(mod.gof= data.frame(vars = mod.list$vars,
                                      dic = NA, dic_vs_base = NA,
                                      waic = NA, waic_vs_base = NA,
                                      mae = NA, mae_vs_base = NA,
                                      rsq = NA, formula = NA),
                  fitted=vector("list", length=nrow(mod.list)),
                  fixed=vector("list", length=nrow(mod.list)),
                  random=vector("list", length=nrow(mod.list)))
  return(mod.out)
}

# run inla models and save outputs as rds
fit.models <- function(mod.out, data=data, base, int, fname=""){
  #  input:
  #    -  mod.out : generated by create.mod.out function with empty output
  #                 (list) except dataframe mod.gof which is populated by 
  #                 variable combinations to run in `vars` col
  #    -  data    : dataframe containing response and covariates (default=data)
  #    -  base    : baseline (null) model object
  #    -  int     : intercept only model object
  #    -  fname   : character string of filename to save rds output
  #  output:
  #    save rds files (1 per 20 models) containing list (mod.out) with elements:
  #    -  mod.gof : dataframe of GOF metrics (as from create.mod.out)
  #    -  fitted  : list of fitted values (length = number of models)
  #    -  fixed   : list of fixed effects (length = number of models)
  #    -  random  : list of random effects (length = number of models)
  #    and return list object (mod.out)
  
  # loop over number of vars in mod.gof
  for (i in 1:nrow(mod.out$mod.gof)){
    # message
    print(paste("Running model",i,"of",nrow(mod.out$mod.gof)))
    
    # assign formula and model
    form <- form.update(mod.out$mod.gof[i, "vars"])
    mod <- runinlamod(formula(form), data)
    
    # save gof metrics
    mod.out$mod.gof[i, "dic"] <- dic(mod)
    mod.out$mod.gof[i, "dic_vs_base"] <- dic.diff(mod, base)
    mod.out$mod.gof[i, "waic"] <- waic(mod)
    mod.out$mod.gof[i, "waic_vs_base"] <- waic.diff(mod, base)
    mod.out$mod.gof[i, "mae"] <- mae(data$cases, mod)
    mod.out$mod.gof[i, "mae_vs_base"] <- mae.diff(data$cases, mod, base)
    mod.out$mod.gof[i, "rsq"] <- rsq(mod, int)
    mod.out$mod.gof[i, "formula"] <- form
    
    # store fitted values, fixed effects and random effects
    mod.out$fitted[[i]] <- mod$summary.fitted.values
    mod.out$fixed[[i]] <- mod$summary.fixed
    mod.out$random[[i]] <- mod$summary.random
    
    # save intermediate results every 1000 models
    if (i %% 1000 == 0) {
      filename <- paste0(fname, "_", i, ".rds")
      saveRDS(mod.out, filename)
    }
  }
  return(mod.out)
}


## Model evaluation functions --------------------------------------------------

# create goodness of fit matrix from mod.out output to support plotting
# each gof metric as raster
gof_matrix <- function(mod.out, temp_var=""){
  # create goodness of fit dataframe
  gof_tmp <- data.frame(temp_lag = rep(1:nlag, floor(nlag/2)^2),
                        spi_long = rep((nlag/2 + 1):nlag, each=(nlag/2) * nlag),
                        spi_short = rep(rep(1:(nlag/2), each=nlag), (nlag/2)),
                        dic = NA, waic = NA, mae = NA, rsq = NA)
  gof_tmp$spi_combo <- paste(gof_tmp$spi_long, gof_tmp$spi_short, sep=".")
  gof_tmp <- gof_tmp %>% relocate(spi_combo, .after=temp_lag)
  
  # goodness of fit variables
  gof_vars <- c("dic", "waic", "mae", "rsq")
  
  # populate goodness of fit dataframe from mod.out output
  for (g in gof_vars){
    for (tlag in 1:6){
      for (spil in 4:6){
        for (spis in 1:3){
          v <- paste0(temp_var, ".", tlag, " * ", "spi6.", spil, " * ", "spi6.",
                      spis)
          
          gof_tmp[gof_tmp$temp_lag == tlag & gof_tmp$spi_long == spil &
                    gof_tmp$spi_short == spis, g] <-
            mod.out$mod.gof[mod.out$mod.gof$vars==v, g]
        }
      }
    }
  }
  
  return(gof_tmp)
}

# calculate the combined covariate fixed effect from model (using mean effect)
model.cce <- function(tmp, idx, lsl_comb, new_col="",
                      spi.long.col="spi6.long", spi.short.col="spi6.short"){
  lsl_comb[[new_col]] <-
    mean(beta.1) * tmp[idx] +
    mean(beta.2) * lsl_comb[[spi.long.col]] +
    mean(beta.3) * lsl_comb[[spi.short.col]] +
    mean(beta.12) * tmp[idx] * lsl_comb[[spi.long.col]] +
    mean(beta.13) * tmp[idx] * lsl_comb[[spi.short.col]] +
    mean(beta.23) * lsl_comb[[spi.long.col]] * lsl_comb[[spi.short.col]] +
    mean(beta.123) * tmp[idx] * lsl_comb[[spi.long.col]] * lsl_comb[[spi.short.col]]
  return(lsl_comb)
}

# calculate 1000 samples of combined covariate fixed effect from model
samples.cce <- function(tmp, idx, s=1000, lsl_comb,
                        spi.long.col="spi6.long", spi.short.col="spi6.short"){
  
  # calculate number of long and short lag combinations
  c <- nrow(lsl_comb)
  
  # create empty matrix with s rows and c columns
  comb_eff <- array(NA,
                    dim=c(s, c),
                    dimnames=list(paste0("s.", 1:s), paste0("comb", 1:c)))
  
  # calculate 1000 combined effects for each long and short lag combo
  comb_eff[1:s, 1:c] <-
    beta.1 %*% t(rep(tmp[idx], c)) +
    beta.2 %*% t(lsl_comb[[spi.long.col]]) +
    beta.3 %*% t(lsl_comb[[spi.short.col]]) +
    beta.12 %*% t(tmp[idx] * lsl_comb[[spi.long.col]]) +
    beta.13 %*% t(tmp[idx] * lsl_comb[[spi.short.col]]) +
    beta.23 %*% t(lsl_comb[[spi.long.col]] * lsl_comb[[spi.short.col]]) +
    beta.123 %*% t(tmp[idx] * lsl_comb[[spi.long.col]] * lsl_comb[[spi.short.col]])
  return(comb_eff)
}

# calculate the combined covariate fixed effect from model (95% credible interval)
ci.cce <- function(tmp, idx, s=1000, lsl_comb, new_col="",
                      spi.long.col="spi6.long", spi.short.col="spi6.short"){
  
  # calculate comb_eff from samples.cce
  comb_eff <- samples.cce(tmp, idx, s, lsl_comb, spi.long.col = "spi6.long", 
                          spi.short.col = "spi6.short")
  
  # calculate 2.5% and 97.5% percentiles of 1000 samples
  for (c in 1:dim(comb_eff)[2]){
    p_025 <- quantile(comb_eff[, c], 0.025)
    p_975 <- quantile(comb_eff[, c], 0.975)
    
    col_025 <- paste0(new_col, ".025")
    col_975 <- paste0(new_col, ".975")
    
    lsl_comb[c, col_025] <- p_025
    lsl_comb[c, col_975] <- p_975
  }
  
  return(lsl_comb)
  
}

# calculate if combined effect credible interval is statistically significant
ss.cce <- function(lsl_comb, col=""){
  
  # set column names
  col_025 <- paste0(col, ".025")
  col_975 <- paste0(col, ".975")
  col_ss <- paste0(col, ".ss")
  
  # calculate if 95% credible interval is statistically significant
  lsl_comb[[col_ss]] <- ifelse(
    (lsl_comb[[col_025]] > 0 & lsl_comb[[col_975]] > 0) |
    (lsl_comb[[col_025]] < 0 & lsl_comb[[col_975]] < 0), TRUE, FALSE
  )
  
  return(lsl_comb)
}


## Plotting functions ----------------------------------------------------------

# function to plot fitted values
plot.fit <- function(mod, title=""){
  mod %>%
    bind_cols(data %>% dplyr::select(date, cases)) %>%
    group_by(date) %>% 
    summarise(cases = sum(cases, na.rm=TRUE),
              mean = sum(mean),
              lower = sum(`0.025quant`),
              upper = sum(`0.975quant`), .groups = "drop") %>%
    mutate(date = as.Date(date)) %>% 
    ggplot(aes(x = date)) +
    geom_ribbon(aes(ymin = lower, ymax = upper), fill = "deeppink2", alpha=0.4) +
    geom_line(aes(y = mean, colour = "Fitted")) +
    geom_line(aes(y = cases, colour = "Observed")) +
    scale_x_date(date_breaks = "1 year", date_labels = "%Y") +
    labs(x = "", y = "", colour = "") +
    scale_color_manual(values = c("Observed" = "#006666", "Fitted" = "deeppink2")) +
    ggtitle(title) +
    theme_bw() +
    theme(axis.text.x = element_text(angle=45, hjust=1), legend.position = "top",
          plot.title = element_text(hjust=0.5), panel.border = element_blank())
}

# plot seasonal random effect
plot.temp.res <- function(mods, titles=""){
  out <- NULL
  for(i in 1:length(mods)){
    add <- mods[[i]][["month"]] %>%
      mutate(type="Seasonal Random Effect",
             time=rep(c(1:12), times=1),
             model=titles[[i]])
    
    out <- bind_rows(out, add)
  }
  
  out %>%
    ggplot() +
    geom_pointrange(aes(y = mean, ymin = `0.025quant`,
                        ymax = `0.975quant`, x = factor(ID),
                        col=model), position=position_dodge(0.3)) +
    scale_x_discrete(labels = c("Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
                                "Jan", "Feb", "Mar", "Apr", "May")) +
    geom_hline(yintercept = 0, linetype = "dashed", col = "grey") +
    facet_wrap(~type, scales = "free") +
    labs(x = "Month of Year", y = expression(delta[m(t)])) +
    theme_classic() +
    theme(axis.text.x = element_text(size=9, hjust=1),
          axis.title = element_text(size=12))
}

# plot yearly random effect
plot.year.res <- function(mods, titles = "", xlabs = ""){
  out <- NULL
  for(i in 1:length(mods)){
    add <- mods[[i]][["year_index"]] %>%
      mutate(type="Interannual Random Effect",
             time=1:nrow(mods[[i]][["year_index"]]),
             model=titles[[i]])
    
    out <- bind_rows(out, add)
  }
  
  out %>%
    ggplot() +
    geom_pointrange(aes(y = mean, ymin = `0.025quant`,
                        ymax = `0.975quant`, x = factor(ID),
                        col = model), position = position_dodge(0.3)) +
    scale_x_discrete(labels = xlabs) +
    geom_hline(yintercept = 0, linetype = "dashed", col = "grey") +
    facet_wrap(~type, scales = "free") +
    labs(x = "Dengue Year", y = expression(gamma[a(t)])) +
    theme_classic() +
    theme(axis.text.x = element_text(size=9, angle=45, hjust=1),
          axis.title = element_text(size=12))
}
 
# heatmap of dic with spi_combo vs temp
heatmap.dic <- function(gof_matrix, var_title=""){
  gof_matrix %>%
    ggplot(aes(x = factor(spi_combo), y = factor(temp_lag), fill = dic)) +
    geom_raster() +
    ylab(paste(var_title, "Temperature Lag")) + 
    scale_fill_gradientn(name = "DIC", colours = rev(brewer.pal(9, "Greens"))) + 
    scale_y_discrete(expand = expansion(0), labels=c(0:5)) +
    scale_x_discrete(expand = expansion(0)) +
    theme_bw() +
    theme(text = element_text(size = 16), axis.title.x=element_blank(),
          axis.title.y = element_text(margin = margin(t = 0, r = 10, b = 0, l = 0)))
}

# heatmap of waic with spi_combo vs temp
heatmap.waic <- function(gof_matrix, var_title=""){
  gof_matrix %>%
    ggplot(aes(x = factor(spi_combo), y = factor(temp_lag), fill = waic)) +
    geom_raster() +
    ylab(paste(var_title, "")) + 
    scale_fill_gradientn(name = "WAIC", colours = rev(brewer.pal(9, "Blues"))) + 
    scale_y_discrete(expand = expansion(0), labels=c(0:5)) +
    scale_x_discrete(expand = expansion(0)) +
    theme_bw() +
    theme(text = element_text(size = 16), axis.title.x=element_blank())
}

# heatmap of mae with spi_combo vs temp
heatmap.mae <- function(gof_matrix, var_title=""){
  gof_matrix %>%
    ggplot(aes(x = factor(spi_combo), y = factor(temp_lag), fill = mae)) +   
    geom_raster() +
    ylab(paste(var_title, "Temperature Lag")) + 
    xlab("SPI-6 Lag Combination") +
    scale_fill_gradientn(name = "MAE",
                         colours = rev(brewer.pal(9, "Oranges"))) + 
    scale_y_discrete(expand = expansion(0), labels=c(0:5)) +
    scale_x_discrete(expand = expansion(0)) +
    theme_bw() +
    theme(text = element_text(size = 16),
          axis.title.x = element_text(margin = margin(t = 20, r = 0, b = 0, l = 0)),
          axis.title.y = element_text(margin = margin(t = 0, r = 10, b = 0, l = 0)))
}

# heatmap of rsq with spi_combo vs temp
heatmap.rsq <- function(gof_matrix, var_title=""){
  gof_matrix %>%
    ggplot(aes(x = factor(spi_combo), y = factor(temp_lag), fill = rsq)) +   
    geom_raster() +
    ylab(paste(var_title, "")) + 
    xlab("SPI-6 Lag Combination") +
    scale_fill_gradientn(name = expression({R[LR]}^2),
                         colours = brewer.pal(9, "Purples")) + 
    scale_y_discrete(expand = expansion(0), labels=c(0:5)) +
    scale_x_discrete(expand = expansion(0)) +
    theme_bw() +
    theme(text = element_text(size = 16),
          axis.title.x = element_text(margin = margin(t = 20, r = 0, b = 0, l = 0)))
}

# plot 4 goodness of fit heatmaps as single plot
multi.heatmap.gof <- function(gof_matrix, title=""){
  print(ggarrange(heatmap.dic(gof_matrix), heatmap.waic(gof_matrix),
                 heatmap.mae(gof_matrix), heatmap.rsq(gof_matrix),
                 ncol = 2, nrow = 2, labels = c("A", "B", "C", "D"), align = "hv",
                 hjust = -0.5, vjust = 1, font.label = list(size = 18))  +
    theme(plot.background = element_rect(fill = "white")))
}

# plot cross-validation fitted vs observed
plot.fit.cv <- function(preds, startyear, startmonth, idx, title=""){
  preds[["outbreak_probability"]][[idx]] %>%
    mutate(cal_year = ifelse((month + startmonth - 1) %% 12 >= startmonth |
                             (month + startmonth - 1) %% 12 == 0,
                             year_index + startyear - 1, year_index + startyear),
           cal_month = ifelse((month + startmonth - 1) %% 12 == 0, 12,
                              (month + startmonth - 1) %% 12),
           date = as.Date(make_date(cal_year, cal_month))) %>% 
    ggplot(aes(x=date)) +
    geom_ribbon(aes(ymin=pred_lower, ymax=pred_upper), alpha=0.25, fill="#FF5C33") +
    geom_line(aes(y=cases), stat="identity", col="black") +
    geom_line(aes(y=pred_mean), col="#FF5C33") +
    labs(x="Month of Year", y="Monthly Case Counts", x="", title=title) +
    scale_x_date(date_breaks="1 year", date_labels="%Y") +
    theme_bw() +
    theme(text = element_text(size = 16), axis.text.x=element_text(hjust=1),
          plot.title=element_text(hjust=0.5), panel.border = element_blank(),
          panel.background = element_blank())
}

# plot cross-validated probability of outbreak
plot.prob.outbreak.cv <- function(preds, startyear, startmonth, idx, 
                                  threshold=0.3, title=""){
  
  yearmin <- min(preds[["outbreak_probability"]][[idx]]$year_index)
  yearmax <- max(preds[["outbreak_probability"]][[idx]]$year_index)
  
  month.lab <- c(month.abb[c(startmonth:12)], month.abb[c(1:(startmonth-1))])
  year.lab <- paste(seq(startyear, startyear + yearmax - yearmin, 1),
                    stringr::str_sub(seq(startyear+1, startyear + yearmax - yearmin + 1, 1),
                                     -2, -1),
                    sep="/")
  
  if (threshold>=0.2 & threshold < 0.23){
    thresh_fill <- c(-Inf, threshold, 0.4, 0.7, 1)
    thresh_labs <- c("[0, 20)","[20,40)","[40,70)", "[70,100]")
  } else if(threshold>=0.23 & threshold < 0.28){
    thresh_fill <- c(-Inf, threshold, 0.4, 0.7, 1)
    thresh_labs <- c(paste0("[0, ", threshold * 100,")"),
                     paste0("[", threshold * 100, ", ", 40, ")"),
                     "[40,70)", "[70,100]")
  } else if(threshold>=0.28 & threshold < 0.33){
    thresh_fill <- c(-Inf, threshold, 0.5, 0.7, 1)
    thresh_labs <- c(paste0("[0, ", threshold * 100,") %  "),
                     paste0("[", threshold * 100, ", ", 50, ") %  "),
                     "[50,70) %  ", "[70,100] %  ")
  } else if(threshold>=0.33 & threshold < 0.38){
    thresh_fill <- c(-Inf, threshold, 0.5, 0.75, 1)
    thresh_labs <- c(paste0("[0, ", threshold * 100,")"),
                     paste0("[", threshold * 100, ", ", 50, ")"),
                     "[55,75)", "[75,100]")
  } else if(threshold>=0.38 & threshold < 0.48){
    thresh_fill <- c(-Inf, threshold, 0.6, 0.8, 1)
    thresh_labs <- c(paste0("[0, ", threshold * 100,")"),
                     paste0("[", threshold * 100, ", ", 60, ")"),
                     "[60,80)", "[80,100]")
  } else if(threshold>=0.48 & threshold <= 0.55){
    thresh_fill <- c(-Inf, threshold, 0.65, 0.8, 1)
    thresh_labs <- c(paste0("[0, ", threshold * 100,")"),
                     paste0("[", threshold * 100, ", ", 65, ")"),
                     "[65,80)", "[80,100]")
  }
  
  preds[["outbreak_probability"]][[idx]] %>%
    mutate(outb = ifelse(obs_outbreak == 1, "X", NA)) %>% 
    ggplot(aes(x = month, y = year_index,
               fill = cut(pred_outbreak_prob, thresh_fill))) +
    geom_raster(alpha = 1) +
    labs(title = title, x = "Month", y = "Year") +
    scale_fill_manual(values = c("#369683", "#FCC873", "#FF7C5C", "#AA1155"),
                      name = "", labels = thresh_labs, drop = FALSE) +
    geom_text(aes(label = outb), colour = "black", fontface = 2, alpha = 2,
              size = 4) +
    scale_y_continuous(breaks = c(yearmin:yearmax), labels = year.lab,
                       expand = expansion(0)) +
    scale_x_continuous(breaks = c(1:12), labels = month.lab,
                       expand = expansion(0)) +
    theme_bw() +
    theme(text = element_text(size = 16), plot.title = element_text(hjust=0.5),
          panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
          legend.position = "top", legend.spacing.x = unit(0.5, 'cm'),
          panel.border = element_blank(), panel.background = element_blank())
}

# plot single roc curve
plot_roc <- function(roc_obj, label){
  
  trigger <- coords(roc_obj, x = "best", best.method = "closest.topleft")
  
  pt <- ggroc(roc_obj) +
    geom_segment(aes(x = 1, xend = 0, y = 0, yend = 1), color = "darkgrey",
                 linetype = "dashed") +
    geom_point(aes(x = trigger[[2]], y = trigger[[3]]), shape = 1, size = 5,
               col = "#FF5C33") +
    geom_text(data = trigger, aes(x = trigger[[2]] + 0.035,
                                  y = trigger[[3]] + 0.035, 
                                  label = paste0((round(trigger[[1]], 2)) * 100,
                                                 "%"))) +
    geom_text(aes(x = 0.3, y = 0.05, label = paste0("AUC: ",
                                                    round(roc_obj$auc, 2)))) +
    labs(x = "1 - False Alarm Rate", y = "True Positive Rate", title = label) +
    theme_bw() +
    theme(text = element_text(size = 16), plot.title = element_text(hjust=0.5),
          panel.border = element_blank(), panel.background = element_blank())
  
  return(pt)
}

# plot two roc curves on same plot with confidence intervals
plot_roc_2 <- function(roc_obj, roc_obj2, ci_obj, ci_obj2, label){
  
  trigger <- coords(roc_obj, x = "best", best.method = "closest.topleft")
  trigger2 <- coords(roc_obj2, x = "best", best.method = "closest.topleft")
  
  pt <- ggroc(list("Selected Model" = roc_obj, "Current Practice" = roc_obj2),
              aes = c("colour")) +
    scale_colour_manual("", 
                        labels = c("Selected", "Baseline"), 
                        values = c("#FF5C33", "turquoise4")) +
    geom_segment(aes(x = 1, xend = 0, y = 0, yend = 1), color = "darkgrey",
                 linetype = "dashed") +
    geom_point(aes(x = trigger[[2]], y = trigger[[3]]), shape = 1, size = 5,
               col = "#FF5C33") +
    geom_text(aes(x = trigger[[2]] + 0.035, y = trigger[[3]] + 0.035,
                  label = paste0((round(trigger[[1]], 2)) * 100, "%")),
              colour="#FF5C33") +
    geom_text(aes(x = 0.1, y = 0.1, label = paste0("AUC: ",
                                                   round(roc_obj$auc, 2))),
              colour = "#FF5C33") +
    geom_point(aes(x = trigger2[[2]], y = trigger2[[3]]), shape = 1, size = 5,
               col = "turquoise4") +
    geom_text(aes(x = trigger2[[2]] + 0.035, y = trigger2[[3]] + 0.035,
                  label = paste0((round(trigger2[[1]], 2)) * 100, "%")),
              colour="turquoise4") +
    geom_text(aes(x = 0.1, y = 0.05, label = paste0("AUC: ",
                                                    round(roc_obj2$auc, 2))),
              colour = "turquoise4") +
    geom_ribbon(data = ci_obj, aes(x = x, ymin = lower, ymax = upper),
                alpha = 0.25, fill = "#FF5C33", inherit.aes = FALSE) +
    geom_ribbon(data = ci_obj2, aes(x = x, ymin = lower, ymax = upper),
                alpha = 0.25, fill = "turquoise4", inherit.aes = FALSE) +
    labs(x = "1 - False Alarm Rate", y = "True Positive Rate", title = label) +
    theme_bw() +
    theme(text = element_text(size = 16), plot.title = element_text(hjust=0.5),
          legend.position = "top", panel.border = element_blank(),
          legend.text = element_text(margin = margin(r = 1, unit = 'cm')),
          panel.background = element_blank())
  
  return(pt)
}

# create gauge plot
# adapted from https://stackoverflow.com/questions/24900903/how-to-draw-gauge-chart-in-r
gg.gauge <- function(pos, breaks = c(0, 30, 50, 70, 100)) {
  require(ggplot2)
  get.poly <- function(a, b, r1 = 0.5, r2 = 1.4) {
    th.start <- pi * (1-a/100)
    th.end   <- pi * (1-b/100)
    th       <- seq(th.start, th.end, length = 100)
    x        <- c(r1 * cos(th), rev(r2 * cos(th)))
    y        <- c(r1 * sin(th), rev(r2 * sin(th)))
    return(data.frame(x, y))
  }
  breaks_labs <- breaks[-length(breaks)] + diff(breaks) / 2
  trigger <- breaks[2]
  
  ggplot() + 
    geom_polygon(data = get.poly(breaks[1], breaks[2]), aes(x,y), fill = "#369683") +
    geom_polygon(data = get.poly(breaks[2], breaks[3]), aes(x,y), fill = "#FCC873") +
    geom_polygon(data = get.poly(breaks[3], breaks[4]), aes(x,y), fill = "#FF7C5C") +
    geom_polygon(data = get.poly(breaks[4], breaks[5]), aes(x,y), fill = "#AA1155") +
    geom_polygon(data = get.poly(pos - 0.65, pos + 0.65, 0.18), aes(x,y), fill = "grey20") +
    geom_text(data = as.data.frame(breaks), size = 4, fontface = "bold", vjust = 0,
              aes(x = 1.5 * cos(pi*(1-breaks/100)), y = 1.5 * sin(pi*(1-breaks/100)),
                  label = paste0(breaks, " %")), colour = "grey40") +
    geom_text(data = as.data.frame(breaks_labs), size = 7, fontface = "bold", vjust = 0,
              aes(x = 0.9 * cos(pi*(1-(breaks_labs)/100)),
                  y = 0.9 * sin(pi*(1-(breaks_labs)/100)),
                  label=c("LOW", "MEDIUM", "HIGH", "VERY HIGH")),
              colour="white") + 
    annotate("text", x = 0, y = 0, label=c(paste0(round(pos), " %")), vjust = 0,
             size = 8, fontface = "bold", colour = "grey20") +
    annotate("segment",
             x = 0.43 * cos(pi*(1-trigger/100)), xend = 0.5 * cos(pi*(1-trigger/100)),
             y = 0.43 * sin(pi*(1-trigger/100)), yend = 0.5 * sin(pi*(1-trigger/100)),
             arrow = arrow(type = "closed", length = unit(0.02, "npc")),
             colour = "grey50") +
    annotate("text",
             x=0.35 * cos(pi*(1-trigger/100)) + 0.21,
             y=0.35 * sin(pi*(1-trigger/100)) + 0.01,
             label="Trigger Threshold", vjust = 0, size = 3.5, fontface = "bold",
             colour = "grey50") +
    labs(x = "Probability of Outbreak", y = "") +
    coord_fixed() +
    theme_bw(base_size = 12) +
    theme(axis.text=element_blank(),
          axis.ticks=element_blank(),
          panel.grid=element_blank(),
          panel.border=element_blank()) 
}


## END