# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## ROLLING-ORIGIN CROSS VALIDATION =============================================

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Martín Lotto Batista  ORCID: 0000-0002-9437-5270
#     Giovenale Moirano     ORCID: 0000-0001-8748-3321
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

out_of_sample <- function(data = NULL,  # dataset
                          response = NULL,  # response variable
                          vars = NULL,  # model covariates
                          res = NULL,  # random effects
                          lags = NULL,  # lead time
                          start_id = 1,  # index to start cross validation
                          save = FALSE,  # save output
                          source_path = getwd(),
                          file_path = getwd(),
                          nsamples = 1000,  # number of posterior samples
                          file_name = "pred",  # file name
                          tmp = TRUE,  # save temporary output
                          temp = NULL){
  
  # read in dependent functions
  source(paste0(source_path, "/run_inla.R"))
  source(paste0(source_path, "/outbreak_probability.R"))
  source(paste0(source_path, "/predictive_ability.R"))
  
  # create directories
  if (file.exists(paste0(file_path, "/tmp"))){
    invisible()
  } else {
    dir.create(paste0(file_path, "/tmp"))
  }
  
  if (file.exists(paste0(file_path, "/prediction_outputs"))){
    invisible()
  } else {
    dir.create(paste0(file_path, "/prediction_outputs"))
  }
  
  # create object to store outputs
  if (tmp){
    temp <- list(posterior_predictive = list(),
                 outbreak_probability = list(),
                 predictive_ability = list())
  }
  
  ntimes <- nrow(data)  # number of months
  npars <- 2  # distribution mean and overdispersion parameter (neg binomial)
  
  # loop over each model  
  for (j in 1:length(vars)) {
    
    v <- vars[[j]]
    l <- lags[[j]]
    
    npreds <- ntimes - start_id + 1
    posterior_predictive <- matrix(NA, npreds, nsamples)
    
    # loop over each month
    for (i in start_id:ntimes){
      
      # remove subsequent information from data
      data.pred <- data[1:i, ]
        
      # remove response from training dataset between forecast issue and target
      data.pred <- data.pred %>% 
        mutate(cases = ifelse(time >= i-l, NA, cases),
               year_index = ifelse(time >= i-l, NA, year_index))

      # run inla model
      form <- reformulate(c(1, na.omit(unlist(res)), na.omit(v)), response)
      m <- run_mod(form, data.pred)
      
      # samples from the posterior marginal distributions of the parameters
      posterior_samples <- inla.posterior.sample(nsamples, m)
      
      # extract samples of parameters of interest
      par_samples <- matrix(NA, npars, nsamples)
      
      for (col in 1:nsamples){
        par_samples[1, col] <- posterior_samples[[col]][["hyperpar"]][[1]]  # overdispersion
        par_samples[2, col] <- posterior_samples[[col]][["latent"]][[i]]  # target month
      }
      
      # posterior predictive distribution
      predictive_out <- array(NA, nsamples)
      
      for(sample in 1:nsamples) {
        par_sample <- par_samples[, sample]
        predictive_out[sample] <- rnbinom(1,
                                          mu = exp(par_sample[-1]), 
                                          size = par_sample[1])
      }
      
      # save posterior predictive
      k <- i - start_id + 1
      posterior_predictive[k, ] <- predictive_out
      
      # month complete
      cat('\r', paste0("Model ", j, " of ", length(vars), " - Target month ", i,
                       " of ", ntimes))
    }
    
    # model complete
    msg <- paste0("Completed model ", j, " of ", length(vars),
                  paste(rep(" ", 15), collapse = ""))
    cat('\r', msg)
    
    # set label
    lab <- ifelse(is.na(v), "no_covars", v)
    
    # save samples from the posterior predictive distribution
    temp[["posterior_predictive"]][[lab]] <- posterior_predictive
    
    # compute outbreak probability
    outbreak_data <- outbreak_probability(data, 
                                          samples = posterior_predictive,
                                          lag = lags[j],
                                          epi_floor = 5,
                                          nyears = length(unique(data[["year_index"]])),
                                          q = 0.75,
                                          response = response,
                                          start_id = start_id)
    
    temp[["outbreak_probability"]][[lab]] <- outbreak_data
    
    # Indicators of predictive ability
    temp[["predictive_ability"]] <- bind_rows(temp[["predictive_ability"]],
                                              predictive_ability(outbreak_data,
                                                                 posterior_predictive,
                                                                 label = lab,
                                                                 response = response))
    
    # Save preliminary output in tmp folder
    saveRDS(temp, paste0(file_path, "/tmp/temp.rds"))
    
  }
  
  if (save) saveRDS(temp, paste0(file_path, "/prediction_outputs/", file_name, ".rds"))
  if (j == length(vars)) unlink(paste0(file_path, "/tmp"), recursive=TRUE)
  
  return(temp)
  
}

## END