# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
## GOODNESS OF FIT =============================================================

# Description:
#     Calculate the goodness of fit metrics for models of increasing complexity
#     that are most closely related to the final selected model, and compare
#     the random effects of similar models.

# Paper:
#     Compound and cascading effects of climatic extremes on dengue risk in the 
#     Caribbean: An impact-based modelling framework using long- and short-lag
#     interactions

# Script authors:
#     Chloe Fletcher        ORCID: 0000-0002-6705-7605
#     Prof. Rachel Lowe     ORCID: 0000-0003-3939-7343

# ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

## Source packages, functions and data -----------------------------------------

# load packages and functions
source("functions/00_packages_functions.R")

# read in harmonised data
data <- read.csv("data/barbados_data.csv")


## Run models -----------------------------------------------------------------

# random effects
re1 <- paste("f(month, model = 'rw2', cyclic = TRUE, constr = TRUE,",
             "scale.model = TRUE, hyper = precision.prior)")
re2 <- "f(year_index, model = 'rw1', hyper = precision.prior)"

# set base formula to intercept only
baseformula <- paste(c("cases ~ 1"), collapse = " + ")

# generate models with intercept + covariates
m0.0 <- runinlamod(formula(baseformula), data)
m0.1 <- runinlamod(formula(form.update("tas.3mon.4")), data)
m0.2 <- runinlamod(formula(form.update("spi6.5")), data)
m0.3 <- runinlamod(formula(form.update("spi6.1")), data)
m0.4 <- runinlamod(formula(form.update("logdir.4")), data)
m0.5 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1")), data)
m0.6 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1")), data)
m0.7 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1 + logdir.4")), data)
m0.8 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data)
m0.9 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + dir.4")), data)

# set base formula to intercept plus seasonal effect
baseformula <- paste(c("cases ~ 1", re1), collapse = " + ")

# generate models with intercept + seasonal + covariates
m1.0 <- runinlamod(formula(baseformula), data)
m1.1 <- runinlamod(formula(form.update("tas.3mon.4")), data)
m1.2 <- runinlamod(formula(form.update("spi6.5")), data)
m1.3 <- runinlamod(formula(form.update("spi6.1")), data)
m1.4 <- runinlamod(formula(form.update("logdir.4")), data)
m1.5 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1")), data)
m1.6 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1")), data)
m1.7 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1 + logdir.4")), data)
m1.8 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data)
m1.9 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + dir.4")), data)

# set base formula to intercept plus seasonal and interannual effects
baseformula <- paste(c("cases ~ 1", re1, re2), collapse = " + ")

# generate models with intercept + seasonal + covariates
m2.0 <- runinlamod(formula(baseformula), data)
m2.1 <- runinlamod(formula(form.update("tas.3mon.4")), data)
m2.2 <- runinlamod(formula(form.update("spi6.5")), data)
m2.3 <- runinlamod(formula(form.update("spi6.1")), data)
m2.4 <- runinlamod(formula(form.update("logdir.4")), data)
m2.5 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1")), data)
m2.6 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1")), data)
m2.7 <- runinlamod(formula(form.update("tas.3mon.4 + spi6.5 + spi6.1 + logdir.4")), data)
m2.8 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + logdir.4")), data)
m2.9 <- runinlamod(formula(form.update("tas.3mon.4 * spi6.5 * spi6.1 + dir.4")), data)

# create empty gof dataframe
name <- c(paste0("m", paste(rep(0:2, each = 10), rep(0:9, 3), sep = ".")))
mods <- mget(name)
vars <- c(paste(rep(c("intercept",
                      "intercept + seasonal",
                      "intercept + seasonal + interannual"), each = 10),
                rep(c("", "tas.3mon.4", "spi6.5", "spi6.1", "logdir.4",
                      "tas.3mon.4 + spi6.5 + spi6.1",
                      "tas.3mon.4 * spi6.5 * spi6.1",
                      "tas.3mon.4 + spi6.5 + spi6.1 + logdir.4",
                      "tas.3mon.4 * spi6.5 * spi6.1 + logdir.4",
                      "tas.3mon.4 * spi6.5 * spi6.1 + dir.4"), 3), sep = " + "))
gof <- data.frame(id = name, mod = vars, dic = NA, waic = NA, mae = NA, rsq = NA)

# tabulate gof for all models
for (m in 1:length(name)){
  gof[m, "dic"] <- dic(mods[[name[m]]])
  gof[m, "waic"] <- waic(mods[[name[m]]])
  gof[m, "mae"] <- mae(data$cases, mods[[name[m]]])
  gof[m, "rsq"] <- rsq(mods[[name[m]]], m0.0)
}


## END
